"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const maybe_1 = require("./maybe");
const result_1 = require("./result");
const testUtils_1 = require("./testUtils");
const throwIncorrectTypeIdentifiedType = () => {
    throw "Incorrectly identified type";
};
const assertUnchangedJust = (monad) => chai_1.expect(monad.get()).to.be.eql(testUtils_1.testNumber1);
const assertUnchangedNone = (monad) => chai_1.expect(monad.get()).to.be.undefined;
const assertFulfilledMapNumberToNumber = (maybe) => chai_1.expect(maybe.get()).to.be.eql(testUtils_1.mapNumbersToNumber(testUtils_1.testNumber1));
const assertFulfilledMapNumberToString = (maybe) => chai_1.expect(maybe.get()).to.be.eql(testUtils_1.mapNumberToString(testUtils_1.testNumber1));
const assertIsJust = (maybe) => chai_1.expect(maybe.isJust()).to.be.true;
const assertIsNone = (maybe) => chai_1.expect(maybe.isNone()).to.be.true;
const assertRejectedMapNumberToNumber = (maybe) => chai_1.expect(maybe.get()).to.be.undefined;
const assertRejectedMapNumberToString = (maybe) => chai_1.expect(maybe.get()).to.be.undefined;
const assertIgnoreOnReject = (maybe) => assertIsJust(maybe) && assertUnchangedJust(maybe);
const assertIgnoreOnFullfil = (maybe) => assertIsNone(maybe) && assertUnchangedNone(maybe);
const nothingToNumber = () => testUtils_1.testNumber2;
describe("Maybe", () => {
    describe("Just", () => {
        const just = maybe_1.Just(testUtils_1.testNumber1);
        describe("isJust()", () => {
            it("function", () => {
                if (maybe_1.isJust(just)) {
                    const value = just.get();
                    chai_1.expect(value).to.be.eql(testUtils_1.testNumber1);
                }
                else
                    throwIncorrectTypeIdentifiedType();
            });
            it("method", () => {
                if (just.isJust()) {
                    const value = just.get();
                    chai_1.expect(value).to.be.eql(testUtils_1.testNumber1);
                }
                else
                    throwIncorrectTypeIdentifiedType();
            });
        });
        describe("isNone()", () => {
            it("function", () => {
                if (maybe_1.isNone(just))
                    throwIncorrectTypeIdentifiedType();
                else {
                    const value = just.get();
                    chai_1.expect(value).to.be.eql(testUtils_1.testNumber1);
                }
            });
            it("method", () => {
                if (just.isNone())
                    throwIncorrectTypeIdentifiedType();
                else {
                    const value = just.get();
                    chai_1.expect(value).to.be.eql(testUtils_1.testNumber1);
                }
            });
        });
        describe("bind()", () => {
            let monad;
            beforeEach(() => {
                monad = maybe_1.Just(testUtils_1.testNumber1);
            });
            describe("onfulfilled mapped to", () => {
                it('value of the same type', () => {
                    const result = monad
                        .bind(testUtils_1.mapNumbersToNumber);
                    assertIsJust(result);
                    return assertFulfilledMapNumberToNumber(result);
                });
                it('value of a different type', () => {
                    const result = monad
                        .bind(testUtils_1.mapNumberToString);
                    assertIsJust(result);
                    return assertFulfilledMapNumberToString(result);
                });
                describe("monad of", () => {
                    it('the same type', () => {
                        const result = monad
                            .bind(value => maybe_1.Just(testUtils_1.mapNumberToString(value)));
                        assertIsJust(result);
                        return assertFulfilledMapNumberToString(result);
                    });
                    it('an opposite type', () => {
                        const result = monad
                            .bind(value => maybe_1.None());
                        assertIsNone(result);
                        return assertRejectedMapNumberToString(result);
                    });
                    it('an different type', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                        const result = monad
                            .bind(value => Promise.resolve(testUtils_1.mapNumberToString(value)));
                        assertIsJust(result);
                        const promise = yield result;
                        const maybe = yield promise;
                        chai_1.expect(maybe).to.be.eql(testUtils_1.mapNumberToString(testUtils_1.testNumber1));
                    }));
                });
            });
            describe("onrejected ignore mapping to", () => {
                describe("monad of", () => {
                    it('the same type', () => {
                        const result = monad
                            .bind(undefined, () => maybe_1.Just(testUtils_1.testNumber2));
                        return assertIgnoreOnReject(result);
                    });
                    it('an opposite type', () => {
                        const result = monad
                            .bind(undefined, () => maybe_1.None());
                        return assertIgnoreOnReject(result);
                    });
                });
            });
        });
    });
    describe("None", () => {
        const none = maybe_1.None();
        describe("isJust()", () => {
            it("function", () => {
                if (maybe_1.isJust(none))
                    throwIncorrectTypeIdentifiedType();
                else
                    chai_1.expect(none.get()).to.be.undefined;
            });
            it("method", () => {
                if (none.isJust())
                    throwIncorrectTypeIdentifiedType();
                else
                    chai_1.expect(none.get()).to.be.undefined;
            });
        });
        describe("isNone()", () => {
            it("function", () => {
                if (maybe_1.isNone(none))
                    chai_1.expect(none.get()).to.be.undefined;
                else
                    throwIncorrectTypeIdentifiedType();
            });
            it("method", () => {
                if (none.isNone())
                    chai_1.expect(none.get()).to.be.undefined;
                else
                    throwIncorrectTypeIdentifiedType();
            });
        });
        describe("bind()", () => {
            let monad;
            beforeEach(() => {
                monad = maybe_1.None();
            });
            describe("onfulfilled ignore mapping to", () => {
                it('value of the same type', () => {
                    const result = monad
                        .bind(testUtils_1.mapNumbersToNumber);
                    assertIgnoreOnFullfil(result);
                });
                it('value of a different type', () => {
                    const result = monad
                        .bind(testUtils_1.mapNumberToString);
                    assertIgnoreOnFullfil(result);
                });
                describe("monad of", () => {
                    it('the same type', () => {
                        const result = monad
                            .bind(value => maybe_1.None());
                        assertIgnoreOnFullfil(result);
                    });
                    it('an opposite type', () => {
                        const result = monad
                            .bind(value => maybe_1.Just(testUtils_1.mapNumbersToNumber(value)));
                        assertIgnoreOnFullfil(result);
                    });
                    it('an different type', () => {
                        const result = monad
                            .bind(value => Promise.resolve(testUtils_1.mapNumbersToNumber(value)));
                        assertIgnoreOnFullfil(result);
                    });
                });
            });
            describe("onrejected mapped to", () => {
                describe("monad of", () => {
                    it('the same monad', () => {
                        const result = monad
                            .bind(undefined, () => maybe_1.None());
                        assertIsNone(result);
                        return assertRejectedMapNumberToNumber(result);
                    });
                    it('an opposite monad', () => {
                        const result = monad
                            .bind(undefined, () => maybe_1.Just(testUtils_1.mapNumbersToNumber(testUtils_1.testNumber1)));
                        assertIsJust(result);
                        return assertFulfilledMapNumberToNumber(result);
                    });
                });
            });
        });
    });
    describe("Maybe with", () => {
        it("value", () => {
            chai_1.expect(maybe_1.Maybe(testUtils_1.testNumber1)).to.be.eql(maybe_1.Just(testUtils_1.testNumber1));
        });
        it("undefined", () => {
            chai_1.expect(maybe_1.Maybe(undefined)).to.be.eql(maybe_1.None());
        });
        describe("isMaybe()", () => {
            it("Just", () => {
                chai_1.expect(maybe_1.isMaybe(maybe_1.Just(testUtils_1.testNumber1))).to.be.true;
            });
            it("None", () => {
                chai_1.expect(maybe_1.isMaybe(maybe_1.None())).to.be.true;
            });
            it("string", () => {
                chai_1.expect(maybe_1.isMaybe("None")).to.be.false;
            });
            it("number", () => {
                chai_1.expect(maybe_1.isMaybe(10)).to.be.false;
            });
            it("boolean", () => {
                chai_1.expect(maybe_1.isMaybe(10)).to.be.false;
            });
            it("object", () => {
                chai_1.expect(maybe_1.isMaybe({})).to.be.false;
            });
            it("Array", () => {
                chai_1.expect(maybe_1.isMaybe(new Array())).to.be.false;
            });
            it("Promise", () => {
                chai_1.expect(maybe_1.isMaybe(Promise.resolve())).to.be.false;
            });
            it("Success", () => {
                chai_1.expect(maybe_1.isMaybe(result_1.Success(testUtils_1.testNumber1))).to.be.false;
            });
            it("Failure", () => {
                chai_1.expect(maybe_1.isMaybe(result_1.Failure(""))).to.be.false;
            });
        });
    });
});
//# sourceMappingURL=maybe.spec.js.map