"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const monad_1 = require("./monad");
const chai_1 = require("chai");
const testUtils_1 = require("./testUtils");
const assertIsCJustSuccess = (result) => chai_1.expect(result).to
    .be
    .instanceOf(monad_1.CJustSuccess);
const assertIsCNoneFailure = (result) => chai_1.expect(result)
    .to
    .be
    .instanceOf(monad_1.CNoneFailure);
const assertUnchangedCJustSuccess = (monad) => monad.then(v => chai_1.expect(v)
    .to
    .eql(testUtils_1.testNumber1));
const assertUnchangedCNoneFailure = (monad) => monad.then(undefined, v => {
    chai_1.expect(v)
        .to
        .eql(testUtils_1.testNumber1);
    return Promise.resolve();
});
const assertIgnoreOnReject = (monad) => {
    assertIsCJustSuccess(monad);
    return assertUnchangedCJustSuccess(monad);
};
const assertIgnoreOnFullfil = (monad) => {
    assertIsCNoneFailure(monad);
    return assertUnchangedCNoneFailure(monad);
};
const assertFulfilledMapNumberToNumber = (monad) => monad.then(v => chai_1.expect(v)
    .to
    .eql(testUtils_1.mapNumbersToNumber(testUtils_1.testNumber1)));
const assertFulfilledMapNumberToString = (monad) => monad.then(v => chai_1.expect(v)
    .to
    .eql(testUtils_1.mapNumberToString(testUtils_1.testNumber1)));
const assertRejectedMapNumberToNumber = (monad) => monad.then(undefined, v => {
    chai_1.expect(v)
        .to
        .eql(testUtils_1.mapNumbersToNumber(testUtils_1.testNumber1));
    return Promise.resolve();
});
const assertRejectedMapNumberToString = (monad) => monad.then(undefined, v => {
    chai_1.expect(v)
        .to
        .eql(testUtils_1.mapNumberToString(testUtils_1.testNumber1));
    return Promise.resolve();
});
describe('CJustSuccess', () => {
    it('get() returns correct primary value', () => {
        chai_1.expect(new monad_1.CJustSuccess(testUtils_1.testNumber1).get()).eql(testUtils_1.testNumber1);
    });
    it('getOrElse() return correct value', () => {
        chai_1.expect(new monad_1.CJustSuccess(testUtils_1.testNumber1).getOrElse(testUtils_1.testNumber2)).eql(testUtils_1.testNumber1);
    });
    it('getOrThrow() return correct value', () => {
        chai_1.expect(new monad_1.CJustSuccess(testUtils_1.testNumber1).getOrThrow()).eql(testUtils_1.testNumber1);
    });
    describe("then()", () => {
        let monad;
        beforeEach(() => {
            monad = new monad_1.CJustSuccess(testUtils_1.testNumber1);
        });
        describe("onfulfilled mapped to", () => {
            it('value of the same type', () => {
                const result = monad
                    .then(testUtils_1.mapNumbersToNumber);
                assertIsCJustSuccess(result);
                return assertFulfilledMapNumberToNumber(result);
            });
            it('value of a different type', () => {
                const result = monad
                    .then(testUtils_1.mapNumberToString);
                assertIsCJustSuccess(result);
                return assertFulfilledMapNumberToString(result);
            });
            describe("monad of", () => {
                it('the same type', () => {
                    const result = monad
                        .then(value => new monad_1.CJustSuccess(testUtils_1.mapNumberToString(value)));
                    assertIsCJustSuccess(result);
                    return assertFulfilledMapNumberToString(result);
                });
                it('an opposite type', () => {
                    const result = monad
                        .then(value => new monad_1.CNoneFailure(testUtils_1.mapNumberToString(value)));
                    assertIsCNoneFailure(result);
                    return assertRejectedMapNumberToString(result);
                });
                it('an different type', () => {
                    const result = monad
                        .then(value => Promise.resolve(testUtils_1.mapNumberToString(value)));
                    chai_1.expect(result)
                        .to
                        .be
                        .instanceOf(Promise);
                    return assertFulfilledMapNumberToString(result);
                });
            });
        });
        describe("onrejected ignore mapping to", () => {
            it('value of the same type', () => {
                const result = monad
                    .then(undefined, testUtils_1.mapNumbersToNumber);
                return assertIgnoreOnReject(result);
            });
            it('value of a different type', () => {
                const result = monad
                    .then(undefined, testUtils_1.mapNumberToString);
                return assertIgnoreOnReject(result);
            });
            describe("monad of", () => {
                it('the same type', () => {
                    const result = monad
                        .then(undefined, value => new monad_1.CJustSuccess(testUtils_1.mapStringToString(value)));
                    return assertIgnoreOnReject(result);
                });
                it('an opposite type', () => {
                    const result = monad
                        .then(undefined, value => new monad_1.CNoneFailure(testUtils_1.mapNumberToString(value)));
                    return assertIgnoreOnReject(result);
                });
                it('an different type', () => {
                    const result = monad
                        .then(undefined, value => Promise.resolve(testUtils_1.mapNumberToString(value)));
                    return assertIgnoreOnReject(result);
                });
            });
        });
    });
});
describe('CNoneFailure', () => {
    it('get() return correct secondary value', () => {
        chai_1.expect(new monad_1.CNoneFailure(testUtils_1.testString1).get()).eql(testUtils_1.testString1);
    });
    it('getOrElse() return fallback value', () => {
        chai_1.expect(new monad_1.CNoneFailure(testUtils_1.testNumber1).getOrElse(testUtils_1.testNumber2)).eql(testUtils_1.testNumber2);
    });
    describe("then()", () => {
        let monad;
        beforeEach(() => {
            monad = new monad_1.CNoneFailure(testUtils_1.testNumber1);
        });
        describe("onfulfilled ignore mapping to", () => {
            it('value of the same type', () => {
                const result = monad
                    .then(testUtils_1.mapNumbersToNumber);
                assertIgnoreOnFullfil(result);
            });
            it('value of a different type', () => {
                const result = monad
                    .then(testUtils_1.mapNumberToString);
                assertIgnoreOnFullfil(result);
            });
            describe("monad of", () => {
                it('the same type', () => {
                    const result = monad
                        .then(value => new monad_1.CNoneFailure(testUtils_1.mapNumberToString(value)));
                    assertIgnoreOnFullfil(result);
                });
                it('an opposite type', () => {
                    const result = monad
                        .then(value => new monad_1.CJustSuccess(testUtils_1.mapNumbersToNumber(value)));
                    assertIgnoreOnFullfil(result);
                });
                it('an different type', () => {
                    const result = monad
                        .then(value => Promise.resolve(testUtils_1.mapNumbersToNumber(value)));
                    assertIgnoreOnFullfil(result);
                });
            });
        });
        describe("onrejected mapped to", () => {
            it('value of the same type', () => {
                const result = monad
                    .then(undefined, testUtils_1.mapNumberToString);
                assertIsCNoneFailure(result);
                return assertRejectedMapNumberToString(result);
            });
            it('value of a different type', () => {
                const result = monad
                    .then(undefined, testUtils_1.mapNumberToString);
                assertIsCNoneFailure(result);
                return assertRejectedMapNumberToString(result);
            });
            describe("monad of", () => {
                it('the same monad', () => {
                    const result = monad
                        .then(undefined, value => new monad_1.CNoneFailure(testUtils_1.mapNumbersToNumber(value)));
                    assertIsCNoneFailure(result);
                    return assertRejectedMapNumberToNumber(result);
                });
                it('an opposite monad', () => {
                    const result = monad
                        .then(undefined, value => new monad_1.CJustSuccess(testUtils_1.mapNumbersToNumber(value)));
                    assertIsCJustSuccess(result);
                    return assertFulfilledMapNumberToNumber(result);
                });
                it('an different monad', () => {
                    const result = monad
                        .then(undefined, value => Promise.resolve(testUtils_1.mapNumberToString(value)));
                    chai_1.expect(result)
                        .to
                        .be
                        .instanceOf(Promise);
                    return assertFulfilledMapNumberToString(result);
                });
            });
        });
    });
});
describe("await", () => {
    describe("CJustSuccess", () => {
        it("function correctly", () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const value = yield new monad_1.CJustSuccess(testUtils_1.testNumber1);
            chai_1.expect(value).to.be.eql(testUtils_1.testNumber1);
        }));
        it("with CNoneFailure", () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const svalue = yield new monad_1.CJustSuccess(testUtils_1.testNumber1);
                const fvalue = yield new monad_1.CNoneFailure(testUtils_1.testString1);
                throw "The line should not be reached";
            }
            catch (e) {
                chai_1.expect(e).to.be.eql(testUtils_1.testString1);
            }
        }));
        it('getOrThrow() throw correct error', () => {
            const error = new Error(testUtils_1.testString1);
            chai_1.expect(() => new monad_1.CNoneFailure(error).getOrThrow()).to.throws(error);
        });
        describe("Promise", () => {
            describe("resolved", () => {
                it("before", () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const pValue = yield Promise.resolve(testUtils_1.testNumber1);
                    const value = yield new monad_1.CJustSuccess(testUtils_1.testNumber2);
                    chai_1.expect(pValue).to.be.eql(testUtils_1.testNumber1);
                    chai_1.expect(value).to.be.eql(testUtils_1.testNumber2);
                }));
                it("after", () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const value = yield new monad_1.CJustSuccess(testUtils_1.testNumber2);
                    const pValue = yield Promise.resolve(testUtils_1.testNumber1);
                    chai_1.expect(pValue).to.be.eql(testUtils_1.testNumber1);
                    chai_1.expect(value).to.be.eql(testUtils_1.testNumber2);
                }));
            });
            describe("rejected", () => {
                it("before", () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    try {
                        const pValue = yield Promise.reject(testUtils_1.testNumber1);
                        const value = yield new monad_1.CJustSuccess(testUtils_1.testNumber2);
                        throw "The line should not be reached";
                    }
                    catch (e) {
                        chai_1.expect(e).to.be.eql(testUtils_1.testNumber1);
                    }
                }));
                it("after", () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    try {
                        const value = yield new monad_1.CJustSuccess(testUtils_1.testString1);
                        const pValue = yield Promise.reject(testUtils_1.testNumber1);
                        throw "The line should not be reached";
                    }
                    catch (e) {
                        chai_1.expect(e).to.be.eql(testUtils_1.testNumber1);
                    }
                }));
            });
        });
    });
    describe("CNoneFailure", () => {
        it("throws value", () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                yield new monad_1.CNoneFailure(testUtils_1.testString1);
                throw "The line should not be reached";
            }
            catch (e) {
                chai_1.expect(e).to.be.eql(testUtils_1.testString1);
            }
        }));
        it("with CJustSuccess", () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const fvalue = yield new monad_1.CNoneFailure(testUtils_1.testString1);
                const svalue = yield new monad_1.CJustSuccess(testUtils_1.testNumber1);
                throw "The line should not be reached";
            }
            catch (e) {
                chai_1.expect(e).to.be.eql(testUtils_1.testString1);
            }
        }));
        describe("Promise", () => {
            describe("resolved", () => {
                it("before", () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    try {
                        const value = yield new monad_1.CNoneFailure(testUtils_1.testNumber2);
                        const pValue = yield Promise.resolve(testUtils_1.testNumber1);
                        throw "The line should not be reached";
                    }
                    catch (e) {
                        chai_1.expect(e).to.be.eql(testUtils_1.testNumber2);
                    }
                }));
                it("after", () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    try {
                        const pValue = yield Promise.resolve(testUtils_1.testNumber1);
                        const value = yield new monad_1.CNoneFailure(testUtils_1.testNumber2);
                        throw "The line should not be reached";
                    }
                    catch (e) {
                        chai_1.expect(e).to.be.eql(testUtils_1.testNumber2);
                    }
                }));
            });
            describe("rejected", () => {
                it("before", () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    try {
                        const pValue = yield Promise.reject(testUtils_1.testNumber1);
                        const value = yield new monad_1.CNoneFailure(testUtils_1.testNumber2);
                        throw "The line should not be reached";
                    }
                    catch (e) {
                        chai_1.expect(e).to.be.eql(testUtils_1.testNumber1);
                    }
                }));
                it("after", () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    try {
                        const value = yield new monad_1.CNoneFailure(testUtils_1.testString1);
                        const pValue = yield Promise.reject(testUtils_1.testNumber1);
                        throw "The line should not be reached";
                    }
                    catch (e) {
                        chai_1.expect(e).to.be.eql(testUtils_1.testString1);
                    }
                }));
            });
        });
    });
});
//# sourceMappingURL=monad.spec.js.map