"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const maybe_1 = require("./maybe");
const result_1 = require("./result");
const testUtils_1 = require("./testUtils");
const testValue = 1;
const errorMsg = "testError";
const throwIncorrectTypeIdentifiedType = () => {
    throw "Incorrectly identified type";
};
const assertUnchangedJust = (monad) => chai_1.expect(monad.get()).to.be.eql(testUtils_1.testNumber1);
const assertUnchangedNone = (monad) => chai_1.expect(monad.get()).to.be.eql(testUtils_1.testString1);
const assertFulfilledMapNumberToNumber = (maybe) => chai_1.expect(maybe.get()).to.be.eql(testUtils_1.mapNumbersToNumber(testUtils_1.testNumber1));
const assertFulfilledMapNumberToString = (maybe) => chai_1.expect(maybe.get()).to.be.eql(testUtils_1.mapNumberToString(testUtils_1.testNumber1));
const assertFulfilledMapStringToString = (maybe) => chai_1.expect(maybe.get()).to.be.eql(testUtils_1.mapStringToString(testUtils_1.testString1));
const assertIsJust = (maybe) => chai_1.expect(maybe.isSuccess()).to.be.true;
const assertIsNone = (maybe) => chai_1.expect(maybe.isFailure()).to.be.true;
const assertRejectedMapStringToString = (maybe) => chai_1.expect(maybe.get()).to.be.eql(testUtils_1.mapStringToString(testUtils_1.testString1));
const assertRejectedMapStringToError = (maybe) => {
    if (maybe.isFailure()) {
        const error = maybe.get();
        chai_1.expect(error).to.be.instanceOf(Error);
        chai_1.expect(error.message).to.be.eql(testUtils_1.testString1);
    }
};
const assertRejectedMapNumberToString = (maybe) => chai_1.expect(maybe.get()).to.be.eql(testUtils_1.testNumber1);
const assertIgnoreOnReject = (maybe) => assertIsJust(maybe) && assertUnchangedJust(maybe);
const assertIgnoreOnFullfil = (maybe) => assertIsNone(maybe) && assertUnchangedNone(maybe);
describe("Result", () => {
    describe("Success", () => {
        const success = result_1.Success(testValue);
        describe("isSuccess()", () => {
            it("function", () => {
                if (result_1.isSuccess(success)) {
                    const value = success.get();
                    chai_1.expect(value).to.be.eql(testValue);
                }
                else
                    throwIncorrectTypeIdentifiedType();
            });
            it("method", () => {
                if (success.isSuccess()) {
                    const value = success.get();
                    chai_1.expect(value).to.be.eql(testValue);
                }
                else
                    throwIncorrectTypeIdentifiedType();
            });
        });
        describe("isFailure()", () => {
            it("function", () => {
                if (result_1.isFailure(success))
                    throwIncorrectTypeIdentifiedType();
                else {
                    const value = success.get();
                    chai_1.expect(value).to.be.eql(testValue);
                }
            });
            it("method", () => {
                if (success.isFailure())
                    throwIncorrectTypeIdentifiedType();
                else {
                    const value = success.get();
                    chai_1.expect(value).to.be.eql(testValue);
                }
            });
        });
        describe("bind()", () => {
            let monad;
            beforeEach(() => {
                monad = result_1.Success(testUtils_1.testNumber1);
            });
            describe("onfulfilled mapped to", () => {
                it('value of the same type', () => {
                    const result = monad
                        .bind(testUtils_1.mapNumbersToNumber);
                    assertIsJust(result);
                    return assertFulfilledMapNumberToNumber(result);
                });
                it('value of a different type', () => {
                    const result = monad
                        .bind(testUtils_1.mapNumberToString);
                    assertIsJust(result);
                    return assertFulfilledMapNumberToString(result);
                });
                describe("monad of", () => {
                    it('the same type', () => {
                        const result = monad
                            .bind(value => result_1.Success(testUtils_1.mapNumberToString(value)));
                        assertIsJust(result);
                        return assertFulfilledMapNumberToString(result);
                    });
                    it('an opposite type', () => {
                        const result = monad
                            .bind(value => result_1.Failure(testUtils_1.mapNumberToString(value)));
                        assertIsNone(result);
                        return assertFulfilledMapNumberToString(result);
                    });
                    it('an different type', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                        const result = monad
                            .bind(value => Promise.resolve(testUtils_1.mapNumberToString(value)));
                        assertIsJust(result);
                        const promise = yield result;
                        const maybe = yield promise;
                        chai_1.expect(maybe).to.be.eql(testUtils_1.mapNumberToString(testUtils_1.testNumber1));
                    }));
                });
            });
            describe("onrejected ignore mapping to", () => {
                it('value of the same type', () => {
                    const result = monad
                        .bind(undefined, testUtils_1.mapStringToString);
                    return assertIgnoreOnReject(result);
                });
                it('value of a different type', () => {
                    const result = monad
                        .bind(undefined, testUtils_1.mapStringToString);
                    return assertIgnoreOnReject(result);
                });
                describe("monad of", () => {
                    it('the same type', () => {
                        const result = monad
                            .bind(undefined, value => result_1.Success(testUtils_1.mapStringToString(value)));
                        return assertIgnoreOnReject(result);
                    });
                    it('an opposite type', () => {
                        const result = monad
                            .bind(undefined, value => result_1.Failure(value));
                        return assertIgnoreOnReject(result);
                    });
                });
            });
        });
    });
    describe("Failure", () => {
        const failure = result_1.Failure(errorMsg);
        describe("isSuccess()", () => {
            it("function", () => {
                if (result_1.isSuccess(failure))
                    throwIncorrectTypeIdentifiedType();
                else
                    chai_1.expect(failure.get()).to.be.eql(errorMsg);
            });
            it("method", () => {
                if (failure.isSuccess())
                    throwIncorrectTypeIdentifiedType();
                else
                    chai_1.expect(failure.get()).to.be.eql(errorMsg);
            });
        });
        describe("isFailure()", () => {
            it("function", () => {
                if (result_1.isFailure(failure))
                    chai_1.expect(failure.get()).to.be.eql(errorMsg);
                else
                    throwIncorrectTypeIdentifiedType();
            });
            it("method", () => {
                if (failure.isFailure())
                    chai_1.expect(failure.get()).to.be.eql(errorMsg);
                else
                    throwIncorrectTypeIdentifiedType();
            });
        });
        describe("bind()", () => {
            let monad;
            beforeEach(() => {
                monad = result_1.Failure(testUtils_1.testString1);
            });
            describe("onfulfilled ignore mapping to", () => {
                it('value of the same type', () => {
                    const result = monad
                        .bind(testUtils_1.mapNumbersToNumber);
                    assertIgnoreOnFullfil(result);
                });
                it('value of a different type', () => {
                    const result = monad
                        .bind(testUtils_1.mapNumberToString);
                    assertIgnoreOnFullfil(result);
                });
                describe("monad of", () => {
                    it('the same type', () => {
                        const result = monad
                            .bind(value => result_1.Failure(testUtils_1.mapNumberToString(value)));
                        assertIgnoreOnFullfil(result);
                    });
                    it('an opposite type', () => {
                        const result = monad
                            .bind(value => result_1.Success(testUtils_1.mapNumbersToNumber(value)));
                        assertIgnoreOnFullfil(result);
                    });
                    it('an different type', () => {
                        const result = monad
                            .bind(value => Promise.resolve(testUtils_1.mapNumbersToNumber(value)));
                        assertIgnoreOnFullfil(result);
                    });
                });
            });
            describe("onrejected mapped to", () => {
                it('value of the same type', () => {
                    const result = monad
                        .bind(undefined, testUtils_1.mapStringToString);
                    assertIsNone(result);
                    return assertRejectedMapStringToString(result);
                });
                it('value of a different type', () => {
                    const result = monad
                        .bind(undefined, value => new Error(value));
                    assertIsNone(result);
                    return assertRejectedMapStringToError(result);
                });
                describe("monad of", () => {
                    it('the same monad', () => {
                        const result = monad
                            .bind(undefined, value => result_1.Failure(testUtils_1.mapStringToString(value)));
                        assertIsNone(result);
                        return assertRejectedMapStringToString(result);
                    });
                    it('an opposite monad', () => {
                        const result = monad
                            .bind(undefined, value => result_1.Success(testUtils_1.mapStringToString(value)));
                        assertIsJust(result);
                        return assertFulfilledMapStringToString(result);
                    });
                });
            });
        });
    });
    describe("Result with", () => {
        it("value", () => {
            chai_1.expect(result_1.Result(() => testValue)).to.be.eql(result_1.Success(testValue));
        });
        it("Success", () => {
            chai_1.expect(result_1.Result(() => result_1.Success(testValue))).to.be.eql(result_1.Success(testValue));
        });
        it("Failure", () => {
            chai_1.expect(result_1.Result(() => result_1.Failure(errorMsg))).to.be.eql(result_1.Failure(errorMsg));
        });
        it("throw", () => {
            chai_1.expect(result_1.Result(() => { throw errorMsg; })).to.be.eql(result_1.Failure(errorMsg));
        });
        describe("isResult()", () => {
            it("Success", () => {
                chai_1.expect(result_1.isResult(result_1.Success(testValue))).to.be.true;
            });
            it("Failure", () => {
                chai_1.expect(result_1.isResult(result_1.Failure(errorMsg))).to.be.true;
            });
            it("string", () => {
                chai_1.expect(result_1.isResult("Failure")).to.be.false;
            });
            it("number", () => {
                chai_1.expect(result_1.isResult(10)).to.be.false;
            });
            it("boolean", () => {
                chai_1.expect(result_1.isResult(10)).to.be.false;
            });
            it("object", () => {
                chai_1.expect(result_1.isResult({})).to.be.false;
            });
            it("Array", () => {
                chai_1.expect(result_1.isResult(new Array())).to.be.false;
            });
            it("Promise", () => {
                chai_1.expect(result_1.isResult(Promise.resolve())).to.be.false;
            });
            it("Just", () => {
                chai_1.expect(result_1.isResult(maybe_1.Just(testValue))).to.be.false;
            });
            it("None", () => {
                chai_1.expect(result_1.isResult(maybe_1.None())).to.be.false;
            });
        });
    });
});
//# sourceMappingURL=result.spec.js.map